
//******************************************************************************
// UPDATE 1/29/98
// Added a return value to CheckLogFileSize. Will return the log file size
// Good for... whatever??  See comments above the function. This changed from
// a procedure to a function.
//
// TCESLogFile is based upon Riccardo 'Rico' Pareschi's (RicoSoft
// rico@mbox.4net.it), component TLogFile. Rico based his component on an idea
// by Song Weng Sam and is a log file component. Needing such a component and not
// wanting to write one myself, I came across Rico's. For my needs, I needed a
// way to create a back-up of the log file when it, the log file, gets too
// large. I also needed one procedure to Open, Write and Close an error or other
// event in one operation. Lastly, since my error messages are all pretty
// standard, I added a design time property to enter a standard error message.
// I also noted a 'bug'. When the procedure CloseLogFile was run, the design time
// LogFile setting was messed up. I added a way to save its value so that the
// component would always remember the log file name that was entered at design
// time. I also rearranged the date and time stamps in the Write procedures.
//
// The design time properties added were ErrorMsg and LogSize.
// Added procedures are: WriteStdErrorMsg, OWCStdErrorMsg and CheckLogFileSize.
//
// This file and component, as are Rico's, are released to the public domain as
// Freeware. You may use it as you may see fit, but I nor any other person who
// has or may have modified this code are responsible should it screw things up
// for you. The basic disclaimer... use at your own risk, but enjoy it!
//
// I also used Rico's resource file as it was a neat Delphi palette icon!
//
// John F. Jarrett    Computer Engineering Services    johnjces@cybertrails.com
// Show Low (named by the turn of a card), Arizona, USA
// www.whitemtns.com/~cesfiles
//
// THANK-YOU Rico!


unit CESLogFile;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, FMXUtils;

type
  TCESLogFile = class(TComponent)

  private
    { Private declarations }
    FLogFile, FStdErrorMsg, TempLogFile, BakName: string;
    FLogSize : Integer;
    LogFx   : text;

  protected
    { Protected declarations }

  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;

    procedure OpenLogFile;
    procedure CloseLogFile;
    procedure WriteLogFile(Msg: string);
    procedure WriteStdErrorMsg;
    procedure OWCStdErrorMsg;
    function CheckLogFileSize : Integer;

  published
    { Published declarations }
    property LogFile: String read FLogFile write FLogFile;
    property StdErrorMsg: String read FStdErrorMsg write FStdErrorMsg;
    property LogSize: Integer read FLogSize write FLogSize;  // JFJ added
  end;

procedure Register;

implementation

const
IsOpen : Boolean = False;

//******************************************************************************
// Register component into the palette
//
procedure Register;
begin
  RegisterComponents('CES', [TCESLogFile]);
end;

//******************************************************************************
// Create the component for design time
//
constructor TCESLogFile.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   FLogFile := '';
   FStdErrorMsg := '';
   FLogSize := 0;
   end;

// *****************************************************************************
// OpenLogFile
// Preserve several variables and open the Log File so it is ready for Writing
// a logged event. The basic extension is '.log'. This is a standard text file
// and can be easily viewed with Notepad. The date and time are written when the
// error logging was started. Flush() ensures that data is written. This could
// placed in the Application/Form OnCreate event.
//
procedure TCESLogFile.OpenLogFile;
begin
 if BakName = '' then
  BakName := FLogFile;
 if TempLogFile = '' then
  begin
   FLogFile := ExtractFilePath(Application.ExeName) + FLogFile + '.log';
   TempLogFile := FLogFile;
  end
  else
   FLogFile := TempLogFile;
   AssignFile(LogFx, FLogFile);
  if FileExists(FLogFile) then
     Append(LogFx)
  else Rewrite(LogFx);
  IsOpen := True;
  Writeln(LogFx, FormatDateTime('"Logging started: " dddd, dd mmmm yyyy - hh:mm', Now));
  Flush(LogFx);
end;

//******************************************************************************
// WriteLogFile
// Write the "log event" with time and a cusotm message, supplied at run time
// then flush.
//
procedure TCESLogFile.WriteLogFile(Msg: string);
begin
  Writeln(LogFx, Msg + ' ' + DateTimeToStr(Now));
  Flush(LogFx);
end;

//******************************************************************************
// WriteStdErrorMsg
// Write the "Standard Error Message which can be set at design time" with time
// added then flush.
//
procedure TCESLogFile.WriteStdErrorMsg;
begin
  Writeln(LogFx, FStdErrorMsg + ' ' + DateTimeToStr(Now));
  Flush(LogFx);
end;

//******************************************************************************
// OWCStdErrorMsg
// A procedure to Open, Write the design time error message and then Close the
// file in one operation. Open-Write-Close  OWCErrorMsg
//
procedure TCESLogFile.OWCStdErrorMsg;
begin
 if IsOpen = False then
 begin
  if BakName = '' then
   BakName := FLogFile;
  if TempLogFile = '' then
   begin
    FLogFile := ExtractFilePath(Application.ExeName) + FLogFile + '.log';
    TempLogFile := FLogFile;
   end
   else
    FLogFile := TempLogFile;
    AssignFile(LogFx, FLogFile);
   if FileExists(FLogFile) then
     Append(LogFx)
   else Rewrite(LogFx);
   FLogFile := TempLogFile;
 end;
   Writeln(LogFx, FStdErrorMsg + ' ' + DateTimeToStr(Now));
   Flush(LogFx);
   CloseFile(LogFx);
   IsOpen := False;
end;

//******************************************************************************
// CloseLogFile
// Closes the log file and writes the date and time of when logging activities
// ended. Puts a blank line between logging activities. A good place for this
// procedure might be in the Application/Form OnClose event.
//
procedure TCESLogFile.CloseLogFile;
begin
  IsOpen := False;
  Writeln(LogFx, FormatDateTime('"Logging ended: " dddd, dd mmmm yyyy - hh:mm', Now));
  Writeln(LogFx, ' ');
  Flush(LogFx);
  CloseFile(LogFx);
  FLogFile := TempLogFile;
end;

//******************************************************************************
// CheckLogFileSize
// Added a CheckLogFileSize proedure. If the file size is greater than
// that speciified, it will back up the current file to FLogFile.bak and
// start a new file. This will only occur when LogSize is greater than 0 and
// when the original file is greater than that entered. Must be called at run-
// time and can be called at anytime after the log file has been opened or
// opened and closed, at least once! Returns log file size, -1 if is does not
// exist or if the log file has not been opened, or opened and closed at least o
// once either by OpenLogFile of OWCStdErrorMsg. This allows your application
// to know whether or not the file has ever been opened for log entry in any
// given instance of your application. Could be a good thing to know or to
// remind your app to use OpenLogFile or to use the one step method
// OWCStdErrorMsg. Once the file has been opened, or opened and closed at
// least once, the return value will be the log file size.
//
function TCESLogFile.CheckLogFileSize : Integer;
var
PBakFileToDelete : Array[0..255] of Char;
WasOpen : Boolean;
begin
 Result := GetFileSize(TempLogFile);
 WasOpen := False;
 If (FLogSize > 0) and (GetFileSize(TempLogFile) > FLogSize) then
  begin
   if IsOpen = True then
    begin
    WasOpen := True;
    CloseLogFile;
    end;
   StrPCopy(PBakFileToDelete, ExtractFilePath(Application.ExeName) + BakName + '.bak');
   DeleteFile(PBakFileToDelete);
   RenameFile(TempLogFile, ExtractFilePath(Application.ExeName) + BakName + '.bak');
   if WasOpen = True then
    OpenLogFile;
  end;
end;

end.
